#ifndef EIGEN_H
#define EIGEN_H
/****************************************************************************/
int eigval (double *wr, double *wi, double *A, int n, int mode) ;
/****************************************************************************/
/*               VALEURS PROPRES D'UNE MATRICE CARREE                      */
/****************************************************************************/
/* Entree :                                                                 */
/*	n	ordre de la matrice (n x n)											*/
/*      A       matrice carree                                              */
/*      mode	indicateur de la methode :                                  */
/*              si mode & 1 => il y aura balancement prealable              */
/*              si mode & 2 => transformations par orthogonalisation (+lent)*/
/*		sinon              //          elementaires (+rapide)				*/
/* Sortie :                                                                 */
/*      eig	0 si convergence dans le calcul des valeurs propres. Sinon		*/
/*              nombre de valeurs calculees avant abandon du calcul.        */
/*      wr,wi   partie reelle et imaginaire des valeurs propres             */
/****************************************************************************/


/****************************************************************************/
int eigvec (double *wr, double *wi, double *M, double *A, int n, int mode) ;
/****************************************************************************/
/*      VALEURS PROPRES ET VECTEURS PROPRES D'UNE MATRICE CARREE            */
/****************************************************************************/
/* Entree :                                                                 */
/*	    n	ordre de la matrice (n x n)										*/
/*      A       matrice carree                                              */
/*      mode	indicateur de la methode :                                  */
/*              si mode & 1 => il y aura balancement prealable              */
/*              si mode & 2 => transformations par orthogonalisation (+lent)*/
/*		sinon              //          elementaires (+rapide)				*/
/*              si mode & 4 => unitarisation des vecteurs propres           */
/* Sortie :                                                                 */
/*      eig	0 si convergence dans le calcul des valeurs propres. Sinon		*/
/*              nombre de valeurs calculees avant abandon du calcul.        */
/*      wr,wi   partie reelle et imaginaire des valeurs propres             */
/*      A	matrice bloc diagonale avec les valeurs propres reelles sur		*/
/*              la diagonale ou un bloc du type    a  b                     */
/*                                                -b  a     si a + i b est  */
/*              valeur propre complexe.                                     */
/*      M       matrice des vecteures propres correspondants. Dans le cas   */
/*              valeurs propres complexes conjugues, M contient d'abord le  */
/*              vecteur partie reelle et apres le vecteur partie imaginaire */
/*              Cette disposition est telle que M * A * inv(M) redonne la   */
/*              matrice A initiale.                                         */
/*                                                                          */
/****************************************************************************/

#endif