/*--------------------------------------------------------------------------*/
/* POLYNOMES.H : Rangement des coefficients degre en ordre croissant.       */
/*--------------------------------------------------------------------------*/
#ifndef POLYNOMES_H
#define POLYNOMES_H
/*--------------------------------------------------------------------------*/
/*       Division euclidienne du polynome a(x) par le polynome b(x).        */
/*--------------------------------------------------------------------------*/
/*       Les coefficients sont ranges en degre croissant.                   */
/*               a(x)   =    b(x)   *    q(x)   -    r(x)                   */
/*    degres :    na          nb      nq=na-nb      nr < nb                 */
/*    q et r et nq sont calcules.                                           */
/*    Renvoi nr degre de r.                                                 */
/*--------------------------------------------------------------------------*/
/*    q et r doivent etre differents de a et b !!!!!!!!!!!!!                */
/*--------------------------------------------------------------------------*/
int divpol (double *a, int na, double *b, int nb, double *q, int *Nq,
								  double *r) ;

/*--------------------------------------------------------------------------*/
/*    Valeur du polynome coef (croissants de degre 0 a n) au point x        */
/*--------------------------------------------------------------------------*/
double valpol (double *coef, int n, double x) ;

/*--------------------------------------------------------------------------*/
/* Valeur derivee du polynome coef (croissants de degre 0 a n) au point x   */
/*--------------------------------------------------------------------------*/
double valderpol (double *coef, int n, double x) ;

/*--------------------------------------------------------------------------*/
/* Polynome derivee du polynome coef (croissants de degre 0 a n)            */
/* Renvoi le degre np-1                                                     */
/*--------------------------------------------------------------------------*/
int derpol (double *coefd, double *coef, int np) ;

/*--------------------------------------------------------------------------*/
/*       Impression du polynome a(x) (coef puissances croissantes)          */
/*--------------------------------------------------------------------------*/
void printpol (double *a, int n) ;

/*--------------------------------------------------------------------------*/
/* Ajoute au polynome coef (croissant) de degre n la racine a. Renvoi n+1   */
/*--------------------------------------------------------------------------*/
int apend_realroot (double *coef, int n, double a) ;

/*--------------------------------------------------------------------------*/
/* Ajoute au pol coef (croissant) de degre n la racine a+ib. Renvoi n+2     */
/*--------------------------------------------------------------------------*/
int apend_cpxroot (double *coef, int n, double a, double b) ;

/*--------------------------------------------------------------------------*/
/* Constitue le polynome caracteristique (croissant) de racines wr+i*wi     */
/*--------------------------------------------------------------------------*/
void rac2pol (double *coefs, double *wr, double *wi, int n) ;


/*--------------------------------------------------------------------------*/
/*  Calcul d'une suite de sturm avec arret au premier facteur commun        */
/*--------------------------------------------------------------------------*/
/* Le tableau A[(M+1)*(M+1)] doit contenir en A[0:M] le polynome a traiter  */
/* Son degre M doit etre indique dans nd[0].                                */
/* La routine calcule le polynomes A1 polynome derivee de A0, puis          */
/* Ai reste de la division euclidienne Ai-2 = Ai-1*Q - Ai pour              */
/* i = 2, jusqu'a l'iteration i = I ou le reste est une constante.          */
/* Les degres des polynomes Ai sont stockes dans nd[i].                     */
/* Si la constante est non nulle, il n'y a pas de facteur commun et on      */
/* renvoi l'index I du dernier polynome reste de degre nd[I] = 0            */
/* Si la constante est nulle, on renvoi l'index I = I-1 du polynome         */
/* precedent qui est facteur commun a P et P' de degre nd[I]                */
/* Remarque : La valeur max de I est nd[0].                                 */
/* Renvoi -1 si echec allocation                                            */
/*--------------------------------------------------------------------------*/
int sturm1 (double *A, int *nd) ;

/*--------------------------------------------------------------------------*/
/*    Calcul du nombre de changements de signes de la suite de Sturm en X   */
/*--------------------------------------------------------------------------*/
/*  Les coefficients en degre croissant du polynome doivent etre ranges     */
/*  dans A0. Son degre M est specifie dans nd[0].                           */
/*  Si I vaut 0, la routine calcule la suite polynomiale de Sturm qui est   */
/*  rangee dans Ai, degre dans nd[i], pour i = 1 a I qui est calcule.       */
/*  Si I est different de 0, c'est que la suite polynomiale de sturm, et I  */
/*  ont deja ete calcules.                                                  */
/*  Le nombre de changements de signes de la suite de sturm en x, pour les  */
/*  racines simples est "returne".                                          */
/*--------------------------------------------------------------------------*/
int sgn_sturm1 (double *A, int *nd, double x, int *I) ;

/*--------------------------------------------------------------------------*/
/*  Calcul d'une suite de sturm jusqu'au reste de degre zero                */
/*--------------------------------------------------------------------------*/
/* Le tableau A[(M+1)*(M+1)] doit contenir en A[0] le polynome a traiter    */
/* Son degre doit etre indique dans nd[0].                                  */
/* La routine calcule le polynome A1 polynome derivee de A0, puis           */
/* Ai reste de la division euclidienne Ai-2 = Ai-1*Q - Ai pour              */
/* i = 2, jusqu'a l'iteration i = I ou le reste est une constante.          */
/* Les degres des polynomes Ai sont stockes dans nd[i].                   */
/* Si la constante est non nulle, il n'y a pas de facteur commun et on      */
/* renvoi l'index I du dernier polynome reste de degre nd[I] = 0            */
/* Si la constante est nulle, on recommence le processus complet a partir   */
/* du polynome precedent qui est facteur commun, et ainsi de suite jusqu'a  */
/* que le reste soit effectivement une constante non nulle ou que le        */
/* degre du dernier facteur commun soit de degre 1.                         */
/* Remarque : La valeur max de I est nd[0].                                 */
/* Renvoi -1 si echec allocation                                            */
/*--------------------------------------------------------------------------*/
int sturmM (int *I0, double *A, int *nd) ;

/*--------------------------------------------------------------------------*/
/*    Calcul du nombre de changements de signes de la suite de Sturm en X   */
/*--------------------------------------------------------------------------*/
/*  Les coefficients en degre croissant du polynome doivent etre ranges     */
/*  dans A0. Son degre M est specifie dans nd[0].                           */
/*  Si I vaut 0, la routine calcule la suite polynomiale de Sturm qui est   */
/*  rangee dans Ai, degre dans nd[i], pour i = 1 a I qui est calcule ainsi  */
/*  que I0.                                                                 */
/*  Si I est different de 0, c'est que la suite polynomiale de sturm, I et  */
/*  I0 ont deja ete calcules.                                               */
/*  Le nombre de changements de signes de la suite de sturm en x, (pour les */
/*  racines simples) est "returne". Celui qui comptabilise les racines      */
/*  multiples est mis dans ntot"                                            */
/*--------------------------------------------------------------------------*/
int sgn_sturmM (int *ntot, double *A, int *nd, double x, int *I,
								      int *I0) ;
/*--------------------------------------------------------------------------*/
/*                 Calcul des racines reelles d'un polynome                 */
/*   Renvoi le nombre de racines calculees ou -1 si echec allocation        */
/*--------------------------------------------------------------------------*/
int roots_sturm (double *RacR, double *Coef, int M, double Epsx) ;

/*--------------------------------------------------------------------------*/
/*   Calcul des racines reelles d'un polynome dans l'intervalle (Xi, Xf)    */
/*   Les racines multiples ne sont comptes qu'une fois                     */
/*   Renvoi le nombre de racines calculees ou -1 si echec allocation        */
/*--------------------------------------------------------------------------*/
int roots_sturmod (double *RacR, double *Coef, int M, 
								double Xi, double Xf, double Epsx) ;

/*--------------------------------------------------------------------------*/
/*  Calcul du nombre de racines reelles d'un polynome dans l'intervalle X,Y */
/*--------------------------------------------------------------------------*/
/*  Les coefficients en degre croissant du polynome doivent etre ranges     */
/*  dans A[0]. Son degre est specifie dans nd[0].                           */
/*  Si I vaut 0, la routine calcule la suite polynomiale de Sturm qui est   */
/*  rangee dans A[i], degre dans nd[i], pour i = 1 a I qui est calcule ainsi*/
/*  que I0.                                                                 */
/*  Si I est different de 0, c'est que la suite polynomiale de sturm, I et  */
/*  I0 ont deja ete calcules.                                               */
/*  A partir de cette suite, par analyse du nombre des changements de signe */
/*  de ces polynomes en X et en Y, on en deduit le nombre total de racines  */
/*  reelles dans cet intervalle -> dans ntot. Le nombre de racines reelles  */
/*  distinctes dans cet intervalle est retourne par le return.              */
/*--------------------------------------------------------------------------*/
int nbracs (int *ntot, double *A, int *nd, double X, double Y,
							      int *I, int *I0) ;

/*--------------------------------------------------------------------------*/
/* Racines d'un polynome par l'algorithme de Newton                         */
/*--------------------------------------------------------------------------*/
/* Calcul de toutes les racines (partie reelle RacR et imaginaire RacI)     */
/* d'un polynome de degre M a coefficients reels.                           */
/* Les coefficientss sont donnes avec les degres dans l'ordre croissant.    */
/* RETOUR : Nb de racines calculees                                         */
/*--------------------------------------------------------------------------*/
/* La methode employee consiste a calculer une racine en complexe a partir  */
/* d'une initialisation complexe proche de zero par la methode de Newton    */
/* Raphson (procedure suivante).                                            */
/* Si elle ne converge pas une initialisation plus grande, dans une autre   */
/* direction, est essayee. Il y a abandon apres plusieurs echecs.           */
/* Lorsqu'il y a convergence, on effectue un abaissement de degre en        */
/* divisant par un polynome du 1er ou 2eme degre suivant que la racine      */
/* trouvee etait reelle ou complexe, et on itere la meme procedure sur le   */
/* polynome de degre inferieur jusqu'a ce qu'il ne reste plus qu'une poly   */
/* de degre 0.                                                              */
/* Pour les polynomes resultant de plusieurs divisions successives, une     */
/* procedure de raffinage de la racine trouvee est entreprise. On essaie de */
/* faire converger le polynome initial (avant toute division) a partir de   */
/* la racine trouvee sur le polynome divise (une seule tentative, avec un   */
/* nombre d'iterations reduit). S'il y a convergence et que la limite       */
/* atteinte n'est pas trop eloignee de la racine trouvee sur le polynome    */
/* divise, on considere que c'est la meme racine, plus precise et elle lui  */
/* est sustituee. On peut en effet reconverger vers une racine differente   */
/* deja traitee, qu'il ne faut pas prendre en compte, car on trouverait une */
/* racine double inexistante.                                               */
/*--------------------------------------------------------------------------*/
int roots_newton (double *RacR, double *RacI, double *Coef, int M) ;


/*--------------------------------------------------------------------------*/
/*        Racines par l'algorithme quotient-difference de RUTISHAUSER       */
/*--------------------------------------------------------------------------*/
int roots_qd (double *RacR, double *RacI, double *Coef, int M) ;
/*--------------------------------------------------------------------------*/
/*	retour  : >= 0 : Nombre de racines calcules							*/
/*	      0	: Pas d'erreur													*/
/*	     -2 : Echec allocation												*/
/*--------------------------------------------------------------------------*/


#endif /* de la portee definie par POLYNOMES_H */
