/************************************************************************/
/* FICHIER QUATERN.H : Utilitaires pour calcul avec les quaternions     */
/************************************************************************/
/* MANIPULATION DE QUATERNIONS avec partage memoire possible */
#ifndef QUATERN_H
#define QUATERN_H


double *qnstd( double *q) ;
/*   quaternion standard : q[0] est rendu positif. */

double *qnprox (double *q, double *qref) ;
/*   proximite de quaternion : q est choisi le plus proche de qref. */

double *cpyqn(double *qs, double *qe) ;
/*   Copie de quaternion : qs = qe */

double *tqn(double *q) ;
/*  Conjugue d'un quaternion */

double *iqn(double *q) ;
/*  Inverse d'un quaternion  */

double *qnqn (double *a, double *b, double *c);
/*   Produit quaternionique : a=b*c  */

double *tqnqn( double *a, double *b, double *c) ;
/*   Produit quaternionique : a=b'*c */

double *qntqn(double *a, double *b, double *c) ;
/*   Produit quaternionique : a=b*c' */

double *iqnqn( double *a, double *b, double *c) ;
/* Produit quaternionique a=inv(b)*c   */

double *qniqn( double *a, double *b, double *c) ;
/* Produit quaternionique a=b*inv(c)   */

double *qnu2mrot (double *A, double *q) ;
/*  Generation de la matrice de passage A associee au quaternion unitaire q */

double *mrot2qn(double *q, double *a) ;
/*  quaternion q correspondant a la matrice A (meme nappe ancien q) */

double *qnu2v3 (double *v3, double *q) ;
/* Vecteur rotation associ au quaternion unitaire q */

double qnu2av3 (double *v3, double *q) ;
/* Vecteur rotation et angle associ au quaternion unitaire q */

double carreqn(double *q) ;
/*  Fournit la norme du quaternion q */

double longqn(double *q) ;
/*  Fournit la longueur du quaternion q */

double lunitqn(double *q) ;
/*  Normalisation a l'unite d'un quaternion */

double *unitqn(double *q) ;
/* Normalisation a l'unite d'un quaternion : Renvoi le quaternion */


double eraqnus(double *qa,double *qb) ;
/*  Val. abs. de l'angle d'ecart rotation entre les 2 quaternions unitaires */

double *dedw(double *deltaq, double *some, double *asom) ;
/*  Calcul de la variation du quaternion par la methode d' Edward2 */

double *omm2dqn (double *dq, double *w, double *q) ;
/*  Derivee du quaternion en fonction de Omega dans Rm : dq=.5*q*w */

double *dqn2omm(double *w, double *dq,double *q) ;
/*   Omega dans Rm a partir de la derivee du quaternion : w = 2.*q'*dq */

double *omo2dqn (double *dq, double *w, double *q) ;
/*  Derivee du quaternion en fonction de Omega dans Ro : dq=.5*w*q */

double *dqn2omo(double *w, double *dq,double *q) ;
/*   Omega dans Ro a partir de la derivee du quaternion : w = 2.*dq*q' */

double *qrotx(double *q, double ang) ;
/* Quaternion de rotation autour de Ox */

double *qroty(double *q, double ang) ;
/* Quaternion de rotation autour de Oy */

double *qrotz(double *q, double ang) ;
/* Quaternion de rotation autour de Oz */

double *qnuv3tqnu(double *a, double *qn, double *b);
/* Rotation d'un vecteur par un quaternion unitaire */

#endif