#ifndef ROTATION_H
#define ROTATION_H
#include "sp3.h"
/**********************************************************************/
/*    UTILITAIRES ROTATION ET DEPLACEMENT DANS R3                     */
/**********************************************************************/
/* Impression d'une por de R3 (3*4 avec translation en 1ere colonne) */
void printpor(char *s, double *a);

double *cpypor(double* u, double* v);

double *porpor(double* a, double* b, double* c);

double *porpori(double* a, double* b, double* c);

double *poripor(double* a, double* b, double* c);

double *porv3(double* a, double* b, double* c);

double *poriv3(double* a, double* b, double* c);

/* Gnration matrice de rotation d'angles d'euler
   Prcession (Oz) , Nutation (Ox1) Rotation propre (Oz2) */
double *euler2mrot(double *a, double *euler);

/* Gnration matrice de rotation d'angles aronautique
   Cap (Oz) , Assiette (Oy1) Roulis (Ox2) */
double *carol2mrot(double *a, double *carol);

/* Gnration matrice de rotation d'angles
   L (Oz) , T (Ox1) R (Oy2) */
double *ltr2mrot(double *a, double *ltr);

/* Gnration matrice de rotation d'angles g autour du
   vecteur numero k (1=X, 2=Y ou 3=Z) */
double *na2mrot(double* a, int k, double g) ;

/* Gnration matrice de rotation d'angles angs autour des
   vecteurs numero i, puis j puis k (1=X, 2=Y ou 3=Z) */
double *angs2mrot(double* m, double* angs, int i, int j, int k);

/* Gnration matrice de rotation d'angle a autour du vecteur V */
double *av3tomrot(double* m, double a, double* v) ;

/* Gnration matrice de rotation autour du vecteur , d'angle gal
	                          la longeur de V */
double *v3tomrot(double* m, double* v) ;

/* Extraction des angles d'Euler : prcession, nutation,
   rotation propre (/Z0,/Y1,/X2) d'une matrice de rotation */
double *mrot2euler(double *euler, double *a);

/* Extraction des angles aero. : Cap, Assiette, Roulis
   (/Z0,/Y1,/X2) d'une matrice de rotation */
double *mrot2carol(double *carol, double *a);

/* Extraction des angles l,t,r (/Z0,/X1,/Y2) d'une matrice de rotation */
double *mrot2ltr (double *ltr, double *A);

/* Extraction des angles Euler-Cardan d'une matrice de rotation */
int mrot2angs(double* angs, double* r, int l, int m, int n) ;

/* Modification des angles Euler-Cardan pour continuite de solution */
void angscont(double *angs, double* angsref, int l, int m, int n) ;

/* Extraction des paramtres d'euler d'une matrice de rotation,
	 savoir la direction (vecteur unitaire) de l'axe de rotation
	unique et de l'angle de rotation correspondante. */
double mrot2av3(double* v, double* a);

/* Extraction de l'angle de rotation d'une matrice de rotation */
double mrot2ang(double* a) ;

/* Extraction du vecteur rotation d'une matrice de rotation,
	 savoir un vecteur de direction l'axe de rotation et de longueur
	l'angle de la rotation correspondante. */
double *mrot2v3(double* v, double* a);

/* Gnration matrice de rotation d'angle a autour de Ox */
double *mrotx(double *m, double a);

/* Gnration matrice de rotation d'angle a autour de Oy */
double *mroty(double *m, double a);

/* Gnration matrice de rotation d'angle a autour de Oz */
double *mrotz(double *m, double a);

/* Composantes Omega dans Rfixe en fonction de la drive des
									angles d'euler */
double *deuler2omo (double *omo, double *deuler, double *euler);

/* Composantes Omega dans Rmobile en fonction de la drive des
									angles d'euler */
double *deuler2omm (double *omm, double *deuler, double *euler);

/* Composantes Omega dans Rfixe en fonction de la drive des
									angles aro. */
double *dcarol2omo (double *omo, double *dcarol, double *carol);

/* Composantes Omega dans Rmobile en fonction de la drive des
									angles aro. */
double *dcarol2omm (double *omm, double *dcarol, double *carol);

/* Composantes Omega dans Rfixe en fonction de la drive des
									angles ltr */
double *dltr2omo (double *omo, double *ltrp, double *ltr);

/* Composantes Omega dans Rmobile en fonction de la drive des
									angles ltr */
double *dltr2omm (double *omm, double *ltrp, double *ltr);

/* Drives des angles d'euler en fonction de Omega dans Ro */
double *omo2deuler (double *deuler, double *omo, double *euler);

/* Drives des angles d'euler en fonction de Omega dans Rm */
double *omm2deuler (double *deuler, double *omm, double *euler);

/* Drives des angles aero en fonction de Omega dans Ro */
double *omo2dcarol (double *dcarol, double *omo, double *carol);

/* Drives des angles aero en fonction de Omega dans Rm */
double *omm2dcarol (double *dcarol, double *omm, double *carol);

/* Drives des angles ltr en fonction de Omega dans Ro */
double *omo2dltr (double *dltr, double *omo, double *ltr);

/* Drives des angles ltr en fonction de Omega dans Rm */
double *omm2dltr (double *dltr, double *omm, double *ltr);
#endif
