#ifndef SPN_H
#define SPN_H
/**********************************************************************/
/*                         UTILITAIRES MATRICIELS                     */
/**********************************************************************/

/***************************************************************************/
/************************  ESPACE DIMENSION N  *****************************/
/***************************************************************************/

void printmat(char *s, double *a, int li, int co);

double *setvec (double* u, double s, int n);

double *cpyvec (double* u, double* v, int n);

double *swapvec (double* u, double* v, int n);

double *negvec (double* u, double* v, int n);

double *addvec (double* u, double* v, double* w, int n);

double *subvec (double* u, double* v, double* w, int n);

double *cumvec (double* u, double* v, double s, double* w, int n);

double *mulvec (double* u, double* v, double* w, int n);

double *divvec (double* u, double* v, double* w, int n);

/* Multiplication d'un vecteur de Rn par un scalaire : U = aV */
double *scavec (double *u, double a, double *v, int n);

double *absvec (double* u, double* v, int n);

/* Composante maximale d'un vecteur de Rn : s = max(Ui) */
double xmaxvec (double* u, int n);

/* Composante max. en val. bas. d'un vecteur de Rn : s = max(|Ui|) */
double xamaxvec (double* u, int n);

/* index du + grand lment (0  n-1) */
int imaxvec (double *u, int n);

/* Composante minimale d'un vecteur de Rn : s = min(Ui) */
double xminvec (double* u, int n);

/* Composante min. en val. abs. d'un vecteur de Rn : s = min(|Ui|) */
double xaminvec (double* u, int n);

/* index du + petit lment (0  n-1) */
int iminvec (double *u, int n);

/* index du + grand lment en valeur absolue (0  n-1) */
int iamaxvec (double *u, int n);

/* index du + petit lment en valeur absolue (0  n-1) */
int iaminvec (double *u, int n);

/* Longueur d'un vecteurs de Rn : s = sqrt(U'*U) */
double longvec (double* u, int n);

/* Normalisation d'un vecteur de Rn : U = V / ||V|| */
double *unitvec (double* u, double *v, int n);

/* Normalisation d'un vecteur de Rn : U = V / ||V|| */
double lunitvec (double* u, double *v, int n);

/* Somme des composantes d'un vecteurs de Rn : s = Sigma Ui */
double xsumvec (double* u, int n);

/* Produit scalaire de vecteurs de Rn : s = <U . V> */
double dotvec (double* u, double* v, int n);

/* Carr d'un vecteurs de Rn : s = U'*U */
double carrevec (double* u, int n);

/* Produit tensoriel de vecteurs de Rn : A = B * C' */
double *tensvec (double *A, double *u, double *v, int nu, int nv);

/* Gnration d'une matrice avec des 1 sur la diagonale */
double *eyemat(double *A, int li, int co);

/* Gnration d'une matrice diagonale A  partir d'un vecteur V */
double *diag2mat(double *A, double *v, int li, int co);

/* Extraction de la diagonale d'une matrice diagonale */
double *mat2diag(double *v, double *A, int li, int co);

/* Gnration colonne contenant la comp. max de chaque ligne */
double *cmaxmat(double *v, double *A, int li, int co) ;

/* Gnration colonne contenant la comp. min de chaque ligne */
double *cminmat(double *v, double *A, int li, int co);

/* Gnration colonne contenant la somme des comp. de chaque ligne */
double *csummat(double *v, double *A, int li, int co);

/* Gnration ligne contenant la comp. max de chaque colonne */
double *lmaxmat(double *v, double *A, int li, int co);

/* Gnration ligne contenant la comp. min de chaque colonne */
double *lminmat(double *v, double *A, int li, int co);

/* Gnration ligne contenant la somme des comp.de chaque colonne */
double *lsummat(double *v, double *A, int li, int co);

/* Produit de matrice : Amp = Bmn x Cnp */
double *matmat (double *A, double *B, double *C, int m, int n, int p); /* !!! A != B ou C */

/* Produit de matrice : Amp = Bmn x [C']np */
double *mattmat (double *A, double *B, double *C, int m, int n, int p); /* !!! A != B ou C */

/* Produit de matrice : Amp = [B']mn x Cnp */
double *tmatmat (double *A, double *B, double *C, int m, int n, int p); /* !!! A != B ou C */

/* Produit de matrice : Amm = [B']mn x Bnm */
void tAAmat (double *a, double *b, int m, int n) ; /* !!! A != B */

/* Produit de matrice : Ann = Bnm x [B']mn */
void AAtmat (double *a, double *b, int n, int m)  ; /* !!! A != B */

/* Trace d'une matrice */
double tracemat(double *A, int li, int co) ;

/* Transpose d'une matrice carre : A = A' */
double *tmat(double *A, int n);

/* Transpose d'une matrice : A(m,n) = B'
   A et B ne doivent pas occuper la mme zone mmoire */
double *trmat(double *A, double *B, int m, int n);
#endif
